@@ -111,14 +111,11 @@ struct GdmGreeterLoginWindowPrivate
 {
         GladeXML        *xml;
         GtkWidget       *user_chooser;
-        GtkWidget       *auth_capslock_label;
         GtkWidget       *auth_banner_label;
         guint            display_is_local : 1;
         guint            is_interactive : 1;
         GConfClient     *client;
 
-        gboolean         caps_lock_on;
-
         gboolean         banner_message_enabled;
         guint            gconf_cnxn;
 
@@ -165,41 +162,6 @@ static void     on_user_unchosen            (GdmUserChooserWidget *user_chooser,
 
 G_DEFINE_TYPE (GdmGreeterLoginWindow, gdm_greeter_login_window, GTK_TYPE_WINDOW)
 
-static void
-capslock_update (GdmGreeterLoginWindow *login_window,
-                 gboolean               is_on)
-{
-
-        login_window->priv->caps_lock_on = is_on;
-
-        if (login_window->priv->auth_capslock_label == NULL) {
-                return;
-        }
-
-        if (is_on) {
-                gtk_label_set_text (GTK_LABEL (login_window->priv->auth_capslock_label),
-                                    _("You have the Caps Lock key on."));
-        } else {
-                gtk_label_set_text (GTK_LABEL (login_window->priv->auth_capslock_label),
-                                    "");
-        }
-}
-
-static gboolean
-is_capslock_on (void)
-{
-        XkbStateRec states;
-        Display     *dsp;
-
-        dsp = GDK_DISPLAY ();
-
-        if (XkbGetState (dsp, XkbUseCoreKbd, &states) != Success) {
-              return FALSE;
-        }
-
-        return (states.locked_mods & LockMask) != 0;
-}
-
 static void
 set_busy (GdmGreeterLoginWindow *login_window)
 {
@@ -581,7 +543,6 @@ switch_mode (GdmGreeterLoginWindow *login_window,
 
         switch (number) {
         case MODE_SELECTION:
-                gtk_widget_set_size_request (GTK_WIDGET (login_window), -1, -1);
                 set_log_in_button_mode (login_window, LOGIN_BUTTON_HIDDEN);
 
                 show_widget (login_window, "cancel-button", FALSE);
@@ -606,9 +567,6 @@ switch_mode (GdmGreeterLoginWindow *login_window,
                 default_name = NULL;
                 break;
         case MODE_AUTHENTICATION:
-                gtk_widget_set_size_request (GTK_WIDGET (login_window),
-                                             GTK_WIDGET (login_window)->allocation.width,
-                                             -1);
                 show_widget (login_window, "cancel-button", TRUE);
                 show_widget (login_window, "shutdown-button", FALSE);
                 show_widget (login_window, "restart-button", FALSE);
@@ -837,7 +795,7 @@ gdm_greeter_login_window_request_timed_login (GdmGreeterLoginWindow *login_windo
         if (login_window->priv->dialog_mode != MODE_SELECTION) {
                 reset_dialog (login_window);
         }
-        gdm_user_chooser_widget_set_show_auto_user (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), TRUE);
+        gdm_user_chooser_widget_set_show_user_auto (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), TRUE);
 
         if (!timed_login_already_enabled) {
                 gdm_user_chooser_widget_set_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser),
@@ -1585,7 +1543,7 @@ load_theme (GdmGreeterLoginWindow *login_window)
         glade_set_custom_handler ((GladeXMLCustomWidgetHandler) custom_widget_constructor,
                                   login_window);
         login_window->priv->xml = glade_xml_new (GLADEDIR "/" GLADE_XML_FILE,
-                                                 "window-box",
+                                                 "window-frame",
                                                  PACKAGE);
 
         g_assert (login_window->priv->xml != NULL);
@@ -1612,7 +1570,7 @@ load_theme (GdmGreeterLoginWindow *login_window)
                 }
         }
 
-        box = glade_xml_get_widget (login_window->priv->xml, "window-box");
+        box = glade_xml_get_widget (login_window->priv->xml, "window-frame");
         gtk_container_add (GTK_CONTAINER (login_window), box);
 
         login_window->priv->user_chooser = glade_xml_get_widget (login_window->priv->xml,
@@ -1641,8 +1599,6 @@ load_theme (GdmGreeterLoginWindow *login_window)
         login_window->priv->auth_banner_label = glade_xml_get_widget (login_window->priv->xml, "auth-banner-label");
         /*make_label_small_italic (login_window->priv->auth_banner_label);*/
 
-        login_window->priv->auth_capslock_label = glade_xml_get_widget (login_window->priv->xml, "auth-capslock-label");
-
         button = glade_xml_get_widget (login_window->priv->xml, "suspend-button");
         g_signal_connect (button, "clicked", G_CALLBACK (suspend_button_clicked), login_window);
 
@@ -1680,7 +1636,6 @@ gdm_greeter_login_window_key_press_event (GtkWidget   *widget,
                                           GdkEventKey *event)
 {
         GdmGreeterLoginWindow *login_window;
-        gboolean               capslock_on;
 
         login_window = GDM_GREETER_LOGIN_WINDOW (widget);
 
@@ -1690,12 +1645,6 @@ gdm_greeter_login_window_key_press_event (GtkWidget   *widget,
                 }
         }
 
-        capslock_on = is_capslock_on ();
-
-        if (capslock_on != login_window->priv->caps_lock_on) {
-                capslock_update (login_window, capslock_on);
-        }
-
         return GTK_WIDGET_CLASS (gdm_greeter_login_window_parent_class)->key_press_event (widget, event);
 }
 
@@ -1703,16 +1652,22 @@ static void
 gdm_greeter_login_window_size_request (GtkWidget      *widget,
                                        GtkRequisition *requisition)
 {
-        int            screen_w;
-        int            screen_h;
-        GtkRequisition child_requisition;
+        int             monitor;
+        GdkScreen      *screen;
+        GtkRequisition  child_requisition;
+        GdkRectangle    area;
 
         if (GTK_WIDGET_CLASS (gdm_greeter_login_window_parent_class)->size_request) {
                 GTK_WIDGET_CLASS (gdm_greeter_login_window_parent_class)->size_request (widget, requisition);
         }
 
-        screen_w = gdk_screen_get_width (gtk_widget_get_screen (widget));
-        screen_h = gdk_screen_get_height (gtk_widget_get_screen (widget));
+        if (!GTK_WIDGET_REALIZED (widget)) {
+                return;
+        }
+
+        screen = gtk_widget_get_screen (widget);
+        monitor = gdk_screen_get_monitor_at_window (screen, widget->window);
+        gdk_screen_get_monitor_geometry (screen, monitor, &area);
 
         gtk_widget_get_child_requisition (GTK_BIN (widget)->child, &child_requisition);
         *requisition = child_requisition;
@@ -1723,8 +1678,12 @@ gdm_greeter_login_window_size_request (GtkWidget      *widget,
         /* Make width be at least 33% screen width
          * and height be at most 80% of screen height
          */
-        requisition->width = MAX (requisition->width, .33 * screen_w);
-        requisition->height = MIN (requisition->height, .80 * screen_h);
+        requisition->width = MAX (requisition->width, .33 * area.width);
+        requisition->height = MIN (requisition->height, .80 * area.height);
+
+       /* Don't ever shrink window width
+        */
+       requisition->width = MAX (requisition->width, widget->allocation.width);
 }
 
 static void
@@ -1968,7 +1927,7 @@ gdm_greeter_login_window_init (GdmGreeterLoginWindow *login_window)
         gtk_window_set_skip_taskbar_hint (GTK_WINDOW (login_window), TRUE);
         gtk_window_set_skip_pager_hint (GTK_WINDOW (login_window), TRUE);
         gtk_window_stick (GTK_WINDOW (login_window));
-        gtk_container_set_border_width (GTK_CONTAINER (login_window), 25);
+        gtk_container_set_border_width (GTK_CONTAINER (login_window), 0);
 
         g_signal_connect (login_window,
                           "window-state-event",
