@@ -148,7 +148,8 @@ get_active_item_id (GdmOptionWidget *widget,
         model = GTK_TREE_MODEL (widget->priv->list_store);
         item_id = NULL;
 
-        if (widget->priv->active_row == NULL) {
+        if (widget->priv->active_row == NULL ||
+            !gtk_tree_row_reference_valid (widget->priv->active_row)) {
                 return NULL;
         }
 
@@ -617,13 +618,10 @@ path_is_separator (GdmOptionWidget *widget,
 static gboolean
 gdm_option_widget_check_visibility (GdmOptionWidget *widget)
 {
-        int number_of_rows;
-
-        number_of_rows = widget->priv->number_of_top_rows +
-                         widget->priv->number_of_middle_rows +
-                         widget->priv->number_of_bottom_rows;
-
-        if (number_of_rows > 1) {
+        if ((widget->priv->number_of_middle_rows != 0) &&
+            (widget->priv->number_of_top_rows > 0 ||
+             widget->priv->number_of_middle_rows > 1 || 
+             widget->priv->number_of_bottom_rows > 0)) {
                 gtk_widget_show (widget->priv->items_combo_box);
                 gtk_widget_show (widget->priv->label);
 
@@ -636,6 +634,7 @@ gdm_option_widget_check_visibility (GdmOptionWidget *widget)
                 gtk_widget_hide (widget->priv->image);
         }
 
+        widget->priv->check_idle_id = 0;
         return FALSE;
 }
 
